//  gscheduler - A General Purpose Project Scheduler
//  Copyright (C) 1999-2000  Brian St. Pierre
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

// command.cpp
//
// This file implements the commands.  This determines
// the application's reaction to user commands (from the menus).
//
// $Id: command.cpp,v 1.8 2000/03/27 03:35:38 bstpierre Exp $
//

#include <ctype.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <list>

using namespace std;

#include <glib.h>

#include "task.h"
#include "reportmgr.hpp"
#include "command.h"
#include "status_bar.h"
#include "task_tree.h"

// For debugging.
#ifdef DEBUG
#define DEBUG_PRINT(msg)        cerr << msg
#else
#define DEBUG_PRINT(msg)
#endif // DEBUG

static Project* g_activeProject = NULL;

extern "C" void
cmd_task_complete(Task* task)
{
    time_t effort;

    // Make sure that a project is selected.
    if(g_activeProject == NULL)
    {
        DEBUG_PRINT("error: no active project\n");
        return;
    }

    task->isComplete = TRUE;

    DEBUG_PRINT("set task to completed: " <<  task->id << endl);
}

extern "C" void
cmd_task_set_info(Task* task, char* description,
                  time_t startDate, time_t endDate, time_t estimate)
{
    time_t effort;

    g_assert(task != NULL);

    // Make sure that a project is selected.
    if(g_activeProject == NULL)
    {
        g_error("error: no active project\n");
        return;
    }

    if(description != NULL)
    {
        // FIXME: Change the label in the task_tree.  This requires
        // somewhat major changes to the task_tree design.
        task_set_description(task, description);
    }

    if(startDate != 0)
    {
        task->startDate = startDate;
    }

    if(endDate != 0)
    {
        task->actualTime = (endDate - startDate);
    }

    if(estimate != 0)
    {
        // TBD: Add auditing of estimates.
        task->estimatedTime = estimate;
    }

    DEBUG_PRINT("cmd_task_set_info " << task->id
                << " description: " << task->description->str
                << " start: " << task->startDate
                << " actual: " << task->actualTime
                << " estimate: " << task->estimatedTime
                << endl);
}

extern "C" void
cmd_report_run(char* reportName)
{
    if(g_activeProject == NULL)
    {
        DEBUG_PRINT("error: no active project\n");
        return;
    }

    if(!ReportMgr::GetReportMgr().RunReport(reportName, g_activeProject))
    {
        DEBUG_PRINT("error: unknown report '" << reportName << "'\n");
        return;
    }

    DEBUG_PRINT("ran report: " << reportName << "\n");
}

extern "C" void
cmd_project_new(char* desc)
{
    // Release previously allocated memory.
    if(g_activeProject != NULL)
    {
        project_free(g_activeProject);
    }

    g_activeProject = project_new(desc);
    task_tree_set_project(g_activeProject);
}

extern "C" void
cmd_task_add(char* desc, time_t StartDate, gdouble numDays)
{
    string timeStr;
    time_t timeEst;
    Task* task;

    // Make sure that a project is selected.
    if(g_activeProject == NULL)
    {
        DEBUG_PRINT("error: no active project\n");
        return;
    }

    // Convert days to seconds.
    // FIXME: Is this conversion (gdouble --> time_t) safe?  Probably not.
    timeEst = (time_t)(numDays * 24 * 60 * 60);

    // Put the new task under management of the taskmgr.
    task = task_new();
    g_string_assign(task->description, desc);
    task->estimatedTime = timeEst;

    project_add_task(g_activeProject, task);

    task_tree_add_task(task);

    DEBUG_PRINT("added task: id " << task->description->str << ", " <<
                desc << ", " << (timeEst / 24 / 60 / 60) << "\n");
}

extern "C" void
cmd_task_reestimate(Task* task, time_t newEstimate)
{
    /* TBD: Don't want to allow GUI to do this directly because
     * we want auditing of estimates (ie. retain a history of all
     * past estimates) at some point in the future. */
    task->estimatedTime = newEstimate;
}

extern "C" void
cmd_task_rename(Task* task, gchar* newDescription)
{
    /* Don't allow changing this via GUI because it is cleaner to
     * go through a "command". */
    g_string_assign(task->description, newDescription);
}

extern "C" void
cmd_task_remove(Task* task)
{
    bool preserveDepyChain;

    // Make sure that a project is selected.
    if(g_activeProject == NULL)
    {
        status_bar_set_message(_("No active project"));
        DEBUG_PRINT("error: no active project\n");
        return;
    }

    /* Remove the task from the UI. */
    task_tree_remove_task(task);

    /* This is hardcoded for now because "true" is unsupported. */
    preserveDepyChain = false;

    /* Remove the task from the project.  Doing this will free
     * the task. */
    project_remove_task(g_activeProject, task->id, preserveDepyChain);
}

extern "C" void
cmd_project_load(char* filename)
{
    ifstream inFile;
    char projDesc[80];

    // Open the input file.
    inFile.open(filename);

    DEBUG_PRINT(__FUNCTION__ << ": opened " << filename << endl);

    // Read the project description.
    inFile.getline(projDesc, sizeof(projDesc));

    // TBD: Need to warn user about destruction of current project.
    if(g_activeProject != NULL)
    {
        project_free(g_activeProject);
    }

    g_activeProject = project_new(projDesc);
    task_tree_set_project(g_activeProject);

    // Read in each of the task descriptions.
    while(!inFile.eof())
    {
        char taskDesc[80];
        time_t estimatedTime;
        time_t actualTime;
        time_t startDate;
        bool isComplete;
        TaskId tid;
        char newline;
        Task* task;

        inFile.getline(taskDesc, sizeof(taskDesc));
        if(strlen(taskDesc) == 0)
        {
            // This is EOF.
            break;
        }

        inFile >> estimatedTime;
        inFile >> actualTime;
        inFile >> startDate;
        inFile >> isComplete;
        inFile >> tid;

        newline = inFile.get();
        if(newline != '\n')
        {
            g_warning("file is corrupted\n");
            return;
        }

        task = task_new();
        g_string_assign(task->description, taskDesc);
        task->estimatedTime = estimatedTime;
        task->id = tid;
        task->actualTime = actualTime;
        task->startDate = startDate;
        task->isComplete = isComplete;

        project_add_task(g_activeProject, task);
        task_tree_add_task(task);
    }
}

extern "C" void
cmd_project_save(char* filename)
{
    GList* tasks;
    GList* iter;
    ofstream outFile;

    if(g_activeProject == NULL)
    {
        // Nothing to do!
        // TBD: Warn user?
        return;
    }

    // Open the input file.
    outFile.open(filename);

    // Read the project description.
    outFile << g_activeProject->description->str << endl;

    tasks = project_get_tasks(g_activeProject);
    for(iter = g_list_first(tasks); iter != NULL; iter = g_list_next(iter))
    {
        Task* task;

        task = (Task*)iter->data;
        outFile << task->description->str << endl;
        outFile << task->estimatedTime << " ";
        outFile << task->actualTime << " ";
        outFile << task->startDate << " ";
        outFile << task->isComplete << " ";
        outFile << task->id << endl;
    }
}

#ifdef DEBUG
extern "C" void
cmd_project_dump()
{
    project_dump(g_activeProject);
}
#endif // DEBUG
