/*  gscheduler - A General Purpose Project Scheduler
**  Copyright (C) 1999-2000  Brian St. Pierre
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
** dlg_task_add.c
**
** This dialog requests initial task information (description, start date,
** estimated time for completion) from the user, creates a new task,
** and adds that task to the database.
**/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <gnome.h>
#include <gtk/gtk.h>

#include "scheduler.h"
#include "command.h"
#include "task.h"
#include "dialogs.h"

struct DlgTaskAdd
{
    GtkWidget* dlg;
    GtkWidget* entDesc;
    GtkWidget* entStartDate;
    GtkWidget* entNumDays;
};

static gint
dlg_task_add_ok(GtkWidget* widget,
                gpointer data)
{
    struct DlgTaskAdd* dlg;
    char* descStr;
    time_t startDate;
    gdouble numDays;

    dlg = (struct DlgTaskAdd*)data;

    /* Get the contents of the text entries. */
    descStr = gtk_editable_get_chars(GTK_EDITABLE(dlg->entDesc), 0, -1);
    startDate = gnome_date_edit_get_date(GNOME_DATE_EDIT(dlg->entStartDate));
    numDays = gnome_number_entry_get_number(
        GNOME_NUMBER_ENTRY(dlg->entNumDays));

    cmd_task_add(descStr, startDate, numDays);

    /* Done with the string from gtk_editable_get_chars. */
    g_free(descStr);

    /* Done, leave the dialog up until the user presses 'close'.
     * (This allows entering multiple new tasks in sequence.)
     * It is helpful if we clear the description, though. */
    gtk_entry_set_text(GTK_ENTRY(dlg->entDesc), _(""));

    return FALSE;
}

static gint
dlg_task_add_close(GtkWidget* widget,
                   gpointer data)
{
    struct DlgTaskAdd* dlg;

    dlg = (struct DlgTaskAdd*)data;

    gtk_widget_destroy(GTK_WIDGET(dlg->entDesc));
    gtk_widget_destroy(GTK_WIDGET(dlg->entStartDate));
    gtk_widget_destroy(GTK_WIDGET(dlg->entNumDays));
    gtk_widget_destroy(GTK_WIDGET(dlg->dlg));

    /* The dialog is closed, free up the entries structure. */
    g_free(dlg);

    return TRUE;
}

void
dlg_task_add(GnomeApp* app)
{
    const int OK_BTN_POS = 0;
    const int CANCEL_BTN_POS = 1;

    GtkWidget* dlg;

    GtkWidget* hbxDesc;
    GtkWidget* hbxDate;
    GtkWidget* hbxNumDays;

    GtkWidget* lblDesc;
    GtkWidget* lblDate;
    GtkWidget* lblNumDays;

    GtkWidget* entDesc;
    GtkWidget* entDate;
    GtkWidget* entNumDays;

    time_t curTime;

    struct DlgTaskAdd* dlgStruct;

    /* NOTE: If adding or moving buttons in this call, must add/change
     * consts above. */
    dlg = gnome_dialog_new(_("Add Task"),
                           GNOME_STOCK_BUTTON_OK,
                           GNOME_STOCK_BUTTON_CLOSE,
                           NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dlg), GTK_WINDOW(app));

    /* Create the entries (with corresponding labels).  Each label/entry
     * combination needs a separate hbox in the vbox. */
    hbxDesc = gtk_hbox_new(FALSE, 0);
    hbxDate = gtk_hbox_new(FALSE, 0);
    hbxNumDays = gtk_hbox_new(FALSE, 0);
    lblDesc = gtk_label_new(_("Description: "));
    lblDate = gtk_label_new(_("Start date: "));
    lblNumDays = gtk_label_new(_("Number of days: "));

    time(&curTime);
    entDesc = gtk_entry_new();
    entDate = gnome_date_edit_new(curTime, 0, 1);
    entNumDays = gnome_number_entry_new(NULL, _("Number of days"));

    dlgStruct = g_malloc(sizeof(struct DlgTaskAdd));
    dlgStruct->dlg = dlg;
    dlgStruct->entDesc = entDesc;
    dlgStruct->entStartDate = entDate;
    dlgStruct->entNumDays = entNumDays;

    gnome_dialog_button_connect(GNOME_DIALOG(dlg), OK_BTN_POS,
                                GTK_SIGNAL_FUNC (dlg_task_add_ok),
                                dlgStruct);
    gnome_dialog_button_connect (GNOME_DIALOG(dlg), CANCEL_BTN_POS,
                                 GTK_SIGNAL_FUNC (dlg_task_add_close),
                                 dlgStruct);

    gtk_box_pack_start(GTK_BOX (hbxDesc), lblDesc,
                       FALSE, FALSE, GTK_PACK_START);
    gtk_box_pack_start(GTK_BOX (hbxDate), lblDate,
                       FALSE, FALSE, GTK_PACK_START);
    gtk_box_pack_start(GTK_BOX (hbxNumDays), lblNumDays,
                       FALSE, FALSE, GTK_PACK_START);

    gtk_box_pack_end(GTK_BOX (hbxDesc), entDesc,
                     TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX (hbxDate), entDate,
                     TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX (hbxNumDays), entNumDays,
                     TRUE, TRUE, 0);

    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 hbxDesc);
    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 hbxDate);
    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 hbxNumDays);

    gtk_widget_show(lblDesc);
    gtk_widget_show(lblDate);
    gtk_widget_show(lblNumDays);
    gtk_widget_show(entDesc);
    gtk_widget_show(entDate);
    gtk_widget_show(entNumDays);
    gtk_widget_show(hbxDesc);
    gtk_widget_show(hbxDate);
    gtk_widget_show(hbxNumDays);

    gtk_widget_show(dlg);
}
