/*  gscheduler - A General Purpose Project Scheduler
**  Copyright (C) 1999-2000  Brian St. Pierre
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
** dlg_task_edit.c
**
** This dialog allows the user to edit task information (description,
** start date, estimated time for completion, completion status) from
** the user and updates that task in the database.
**
** $Id: dlg_task_edit.c,v 1.1 2000/03/27 03:35:38 bstpierre Exp $
**/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#include <gnome.h>
#include <gtk/gtk.h>

#include "scheduler.h"
#include "command.h"
#include "task.h"
#include "dialogs.h"

struct DlgTaskEdit
{
    GtkWidget* dlg;
    GtkWidget* entDesc;
    GtkWidget* entStartDate;
    GtkWidget* entEndDate;
    GtkWidget* entNumDays;
    GtkWidget* chkCompleted;
    Task* task;
};

static gint
dlg_task_edit_ok(GtkWidget* widget,
                 gpointer data)
{
    struct DlgTaskEdit* dlg;
    gboolean isComplete;
    char* descStr;
    time_t startDate;
    time_t endDate;
    time_t newEstimate;
    gdouble numDays;
    Task* task;

    dlg = (struct DlgTaskEdit*)data;
    task = dlg->task;

    /* Get the contents of the text entries. */
    descStr = gtk_editable_get_chars(GTK_EDITABLE(dlg->entDesc), 0, -1);
    startDate = gnome_date_edit_get_date(GNOME_DATE_EDIT(dlg->entStartDate));
    endDate = gnome_date_edit_get_date(GNOME_DATE_EDIT(dlg->entEndDate));
    numDays = gnome_number_entry_get_number(
        GNOME_NUMBER_ENTRY(dlg->entNumDays));

    if(strcmp(descStr, task->description->str) == 0)
    {
        /* Don't need to update the task; this saves some effort. */
        g_free(descStr);
        descStr = NULL;
    }

    if(numDays == task->estimatedTime)
    {
        /* Don't need to update the time estimate; this saves some
         * effort and doesn't throw off the audting. */
        newEstimate = 0;
    }
    else
    {
        newEstimate = numDays * 60 * 60 * 24;
    }

    cmd_task_set_info(task, descStr, startDate, endDate, newEstimate);

    /* If the user checked the "completed" button, then we need to
     * mark the task completed. */
    isComplete = gtk_toggle_button_get_active(
        GTK_TOGGLE_BUTTON(dlg->chkCompleted));
    if(isComplete)
    {
        /* FIXME: need real numbers for startDate, numDays. */
        cmd_task_complete(task);
    }

    gtk_widget_destroy(GTK_WIDGET(dlg->entDesc));
    gtk_widget_destroy(GTK_WIDGET(dlg->entStartDate));
    gtk_widget_destroy(GTK_WIDGET(dlg->entEndDate));
    gtk_widget_destroy(GTK_WIDGET(dlg->entNumDays));
    gtk_widget_destroy(GTK_WIDGET(dlg->chkCompleted));
    gtk_widget_destroy(GTK_WIDGET(dlg->dlg));

    /* The dialog is closed, free up the entries structure. */
    g_free(dlg);

    return TRUE;
}

static gint
dlg_task_edit_close(GtkWidget* widget,
                    gpointer data)
{
    struct DlgTaskEdit* dlg;

    dlg = (struct DlgTaskEdit*)data;

    gtk_widget_destroy(GTK_WIDGET(dlg->entDesc));
    gtk_widget_destroy(GTK_WIDGET(dlg->entStartDate));
    gtk_widget_destroy(GTK_WIDGET(dlg->entEndDate));
    gtk_widget_destroy(GTK_WIDGET(dlg->entNumDays));
    gtk_widget_destroy(GTK_WIDGET(dlg->chkCompleted));
    gtk_widget_destroy(GTK_WIDGET(dlg->dlg));

    /* The dialog is closed, free up the entries structure. */
    g_free(dlg);

    return TRUE;
}

void dlg_task_edit(GnomeApp* app, Task* task)
{
    const int OK_BTN_POS = 0;
    const int CANCEL_BTN_POS = 1;

    struct DlgTaskEdit* dlgStruct;
    time_t curTime;

    GtkWidget* dlg;

    GtkWidget* hbxDesc;
    GtkWidget* hbxStart;
    GtkWidget* hbxEnd;
    GtkWidget* hbxNumDays;

    GtkWidget* lblDesc;
    GtkWidget* lblStart;
    GtkWidget* lblEnd;
    GtkWidget* lblNumDays;

    GtkWidget* entDesc;
    GtkWidget* entStart;
    GtkWidget* entEnd;
    GtkWidget* entNumDays;

    GtkWidget* chkCompleted;

    /* NOTE: If adding or moving buttons in this call, must add/change
     * consts above. */
    dlg = gnome_dialog_new(_("Edit Task"),
                           GNOME_STOCK_BUTTON_OK,
                           GNOME_STOCK_BUTTON_CANCEL,
                           NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(dlg), GTK_WINDOW(app));

    /*  Each label/entry combination needs a separate hbox in the vbox. */
    hbxDesc = gtk_hbox_new(FALSE, 0);
    hbxStart = gtk_hbox_new(FALSE, 0);
    hbxEnd = gtk_hbox_new(FALSE, 0);
    hbxNumDays = gtk_hbox_new(FALSE, 0);
    lblDesc = gtk_label_new(_("Description: "));
    lblStart = gtk_label_new(_("Start date: "));
    lblEnd = gtk_label_new(_("End date: "));
    lblNumDays = gtk_label_new(_("Estimated days: "));
    chkCompleted = gtk_check_button_new_with_label("Completed");

    /* Create the entries (with corresponding labels). */
    time(&curTime);
    entDesc = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(entDesc), task->description->str);
    if(task->startDate == 0)
    {
        entStart = gnome_date_edit_new(curTime, 0, 1);
    }
    else
    {
        entStart = gnome_date_edit_new(task->startDate, 0, 1);
    }

    if(task->actualTime == 0)
    {
        entEnd = gnome_date_edit_new(curTime, 0, 1);
    }
    else
    {
        entEnd = gnome_date_edit_new(task->startDate + task->actualTime, 0, 1);
    }

    entNumDays = gnome_number_entry_new(NULL, _("Estimated days"));
    {
        GtkEntry* gtkEnt;
        GnomeNumberEntry* gnomeNumEnt;
        gchar* numDaysStr;
        guint numDays;

        numDays = task->estimatedTime / 60 / 60 / 24;
        numDaysStr = g_strdup_printf("%d", numDays);

        gnomeNumEnt = GNOME_NUMBER_ENTRY(entNumDays);
        gtkEnt = GTK_ENTRY(gnome_number_entry_gtk_entry(gnomeNumEnt));
        gtk_entry_set_text(gtkEnt, numDaysStr);
    }

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkCompleted),
                                 task->isComplete);

    dlgStruct = g_malloc(sizeof(struct DlgTaskEdit));
    dlgStruct->dlg = dlg;
    dlgStruct->entDesc = entDesc;
    dlgStruct->entStartDate = entStart;
    dlgStruct->entEndDate = entEnd;
    dlgStruct->entNumDays = entNumDays;
    dlgStruct->task = task;
    dlgStruct->chkCompleted = chkCompleted;

    gnome_dialog_button_connect(GNOME_DIALOG(dlg), OK_BTN_POS,
                                GTK_SIGNAL_FUNC (dlg_task_edit_ok),
                                dlgStruct);
    gnome_dialog_button_connect (GNOME_DIALOG(dlg), CANCEL_BTN_POS,
                                 GTK_SIGNAL_FUNC (dlg_task_edit_close),
                                 dlgStruct);

    gtk_box_pack_start(GTK_BOX (hbxDesc), lblDesc,
                       FALSE, FALSE, GTK_PACK_START);
    gtk_box_pack_start(GTK_BOX (hbxStart), lblStart,
                       FALSE, FALSE, GTK_PACK_START);
    gtk_box_pack_start(GTK_BOX (hbxEnd), lblEnd,
                       FALSE, FALSE, GTK_PACK_START);
    gtk_box_pack_start(GTK_BOX (hbxNumDays), lblNumDays,
                       FALSE, FALSE, GTK_PACK_START);

    gtk_box_pack_end(GTK_BOX (hbxDesc), entDesc,
                     TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX (hbxStart), entStart,
                     TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX (hbxEnd), entEnd,
                     TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX (hbxNumDays), entNumDays,
                     TRUE, TRUE, 0);

    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 hbxDesc);
    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 hbxStart);
    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 hbxEnd);
    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 hbxNumDays);
    gtk_box_pack_start_defaults (GTK_BOX (GNOME_DIALOG (dlg)->vbox),
                                 chkCompleted);

    gtk_widget_show(lblDesc);
    gtk_widget_show(lblStart);
    gtk_widget_show(lblEnd);
    gtk_widget_show(lblNumDays);

    gtk_widget_show(entDesc);
    gtk_widget_show(entStart);
    gtk_widget_show(entEnd);
    gtk_widget_show(entNumDays);

    gtk_widget_show(hbxDesc);
    gtk_widget_show(hbxStart);
    gtk_widget_show(hbxEnd);
    gtk_widget_show(hbxNumDays);
    gtk_widget_show(chkCompleted);

    gtk_widget_show(dlg);
}
