/*  gscheduler - A General Purpose Project Scheduler
**  Copyright (C) 1999-2000  Brian St. Pierre
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* project.h
**
** This file implements the details of the "Project" encapsulation.
**
** $Log: project.c,v $
** Revision 1.4  2000/02/26 06:47:33  bstpierre
** Implemented Task->Remove
**
** Revision 1.3  2000/02/26 05:22:26  bstpierre
** Finished task12064: Clean up interface between UI and back end.
**
** Revision 1.2  2000/02/26 03:23:58  bstpierre
** Finished task11582: "Add a status bar"
** Finished task11573: "Put full task information into the task tree"
**
** Revision 1.1  2000/02/25 05:13:05  bstpierre
** * Changed Project from class to struct; project.cpp to project.c
** * Changed task_tree_set_project to take Project object instead of description
** * Fixed bugs in Task object introduced by last checkin.
** * Added Debug menu that is conditionally compiled (-DDEBUG to use).
** * Fixed up a couple of command APIs
**
**/

#include <glib.h>
#include "project.h"

Project*
project_new(gchar* description)
{
    Project* project;

    g_assert(description != NULL);

    /* Allocate a new project object on the heap. */
    project = g_new(Project, 1);

    /* A project's first task id will be 1. */
    project->nextTaskId = 1;

    /* To initialize a GList, set the pointer to NULL. */
    project->tasks = NULL;

    project->description = g_string_new(description);

    return project;
}

void
project_free(Project* project)
{
    g_assert(project != NULL);

    g_string_free(project->description, TRUE);
    project->description = NULL;
    g_free(project);
}

void
project_add_task(Project* project, Task* task)
{
    g_assert(project != NULL);
    g_assert(task != NULL);

    task->id = project->nextTaskId;
    project->nextTaskId++;

    project->tasks = g_list_append(project->tasks, task);

#ifdef DEBUG
    g_print("Project '%s' added task '%s / %d', now %d tasks\n",
            project->description->str, task->description->str,
            task->id, g_list_length(project->tasks));
#endif
}

void
project_remove_task(Project* project,
                    TaskId tid,
                    gboolean preserveDepyChain)
{
    GList* iter;
    Task* task;

    g_assert(project != NULL);

    /* This feature is currently unimplemented. */
    g_assert(!preserveDepyChain);

    iter = g_list_first(project->tasks);
    while(iter != NULL)
    {
        task = iter->data;
        if(task->id == tid)
        {
            break;
        }

        iter = g_list_next(iter);
    }

    if(iter == NULL)
    {
        /* Not found. */
        return;
    }
    else
    {
        task = iter->data;
        task_free(task);

        project->tasks = g_list_remove_link(project->tasks, iter);
        g_list_free_1(iter);
    }
}

Task*
project_find_task_by_id(const Project* project, TaskId tid)
{
    GList* iter;
    Task* task;

    g_assert(project != NULL);

    iter = g_list_first(project->tasks);
    while(iter != NULL)
    {
        task = iter->data;
        if(task->id == tid)
        {
            break;
        }

        iter = g_list_next(iter);
    }

    if(iter == NULL)
    {
        // Not found.
        return NULL;
    }
    else
    {
        return task;
    }
}

GList*
project_get_tasks(const Project* project)
{
    g_assert(project != NULL);

    return project->tasks;
}

#ifdef DEBUG
void
project_dump(const Project* project)
{
    GList* iter;

    if(project == NULL)
    {
        g_print("No current project\n");
        return;
    }

    g_print("Project: %s, %d tasks\n",
            project->description->str,
            g_list_length(project->tasks));
    iter = g_list_first(project->tasks);
    while(iter != NULL)
    {
        Task* task;

        task = iter->data;
        g_print("\tTask: %s %d\n", task->description->str, task->id);

        iter = g_list_next(iter);
    }
}
#endif // DEBUG
