/*  gscheduler - A General Purpose Project Scheduler
**  Copyright (C) 1999-2000  Brian St. Pierre
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
** task_tree.c
**
** This file implements the GtkTree that holds the active project's tasks.
**
** $Log: task_tree.c,v $
** Revision 1.7  2000/03/27 03:35:38  bstpierre
** Finished task11574: edit task dialog
**
** Revision 1.6  2000/02/26 06:47:33  bstpierre
** Implemented Task->Remove
**
** Revision 1.5  2000/02/26 05:22:26  bstpierre
** Finished task12064: Clean up interface between UI and back end.
**
** Revision 1.4  2000/02/26 03:23:58  bstpierre
** Finished task11582: "Add a status bar"
** Finished task11573: "Put full task information into the task tree"
**
**/

#include <assert.h>
#include <gnome.h>
#include "scheduler.h"
#include "status_bar.h"
#include "task_tree.h"

static GtkWidget* g_tree = NULL;
static GtkWidget* g_projectTree = NULL;
static GtkWidget* g_curSelection = NULL;
static Project* g_project = NULL;
static Task* g_task = NULL;

static void callback_select_project(GtkTreeItem *tree, Project* project);
static void callback_select_task(GtkTreeItem *tree, Task* task);
static void callback_deselect(GtkTreeItem *tree, gpointer dummy);

void task_tree_init(GnomeApp* app)
{
    if(g_tree != NULL)
    {
        /* Already initialized. */
        return;
    }

    /* Create a tree to display the tasks. */
    g_tree = gtk_tree_new();
    gtk_tree_set_view_lines(GTK_TREE(g_tree), FALSE);
    gtk_tree_set_selection_mode(GTK_TREE(g_tree),
                                GTK_SELECTION_SINGLE);

    /* Add the tree to the app. */
    gnome_app_set_contents(GNOME_APP(app), g_tree);
}

void task_tree_clear()
{
    GList* items;

    if(g_tree == NULL)
    {
        /* Not initialized yet. */
        return;
    }

    /* First unselect the current selection.  This is necessary
     * because the remove_items below doesn't unset the current selection.
     * If an item is selected when we remove them, we will get an assertion
     * failure when the user selects whatever is next added to the 
     * remaining tree. */
    /* TBD: There is probably a better way of doing this -- maybe
     * by handling the "selection-changed" signal. */
    gtk_tree_select_item(GTK_TREE(g_projectTree), 0);
    gtk_tree_unselect_item(GTK_TREE(g_projectTree), 0);

    items = gtk_container_children (GTK_CONTAINER(g_tree));
    if(g_list_length(items) > 0)
    {
        /* Only remove items if the list has elements in it. */
        gtk_tree_remove_items(GTK_TREE(g_tree), items);
    }

    /* This tree's ref count goes to zero from the remove_items above.
     * Set to NULL to stay in touch with reality elsewhere. */
    g_projectTree = NULL;

    /* Using GTK_CONTAINER creates a duplicate list, need to free it. */
    g_list_free(items);
}

void task_tree_set_project(Project* project)
{
    GtkWidget* treeItem;

    assert(g_tree != NULL);

    if(g_projectTree != NULL)
    {
        task_tree_clear();
    }

    /* Remember this project for future use. */
    g_project = project;

    treeItem = gtk_tree_item_new_with_label(project->description->str);
    gtk_signal_connect(GTK_OBJECT(treeItem),
                      "select" ,
                       GTK_SIGNAL_FUNC(callback_select_project),
                       project);
    gtk_signal_connect(GTK_OBJECT(treeItem),
                      "deselect" ,
                       GTK_SIGNAL_FUNC(callback_deselect),
                       0);
    gtk_widget_show(treeItem);
    gtk_tree_append(GTK_TREE(g_tree), treeItem);

    /* Place tasks on a subtree of the project item. */
    g_projectTree = gtk_tree_new();
    gtk_tree_set_view_lines(GTK_TREE(g_projectTree), FALSE);
    gtk_tree_item_set_subtree(GTK_TREE_ITEM(treeItem), g_projectTree);

    status_bar_set_project(g_project);
}

void task_tree_add_task(Task* task)
{
    GtkWidget* treeItem;
    gchar* taskLabel;

    g_assert(g_projectTree != NULL);
    g_assert(task != NULL);
    g_assert(task->description != NULL);
    g_assert(task->description->str != NULL);

    taskLabel = g_strdup_printf("%d\t: %s", task->id, task->description->str);
    treeItem = gtk_tree_item_new_with_label(taskLabel);
    gtk_signal_connect(GTK_OBJECT(treeItem),
                      "select" ,
                       GTK_SIGNAL_FUNC(callback_select_task),
                       task);
    gtk_signal_connect(GTK_OBJECT(treeItem),
                      "deselect" ,
                       GTK_SIGNAL_FUNC(callback_deselect),
                       0);
    gtk_tree_append(GTK_TREE(g_projectTree), treeItem);
    gtk_widget_show(treeItem);

    status_bar_set_message(taskLabel);
    g_free(taskLabel);
}

Task* task_tree_get_current_task()
{
    return g_task;
}

void task_tree_remove_task(Task* task)
{
    GtkLabel* taskLabel;
    gchar* labelString;
    GList* iter;

    g_assert(task != NULL);

    if(task == g_task)
    {
        gchar* message;
        GtkWidget* itemToRemove;

        g_assert(g_curSelection != NULL);

        itemToRemove = g_curSelection;

        /* Must unselect the child before removing or we will think
         * there is still a current selection. */
        gtk_tree_unselect_child(GTK_TREE(g_projectTree),
                                g_curSelection);

        /* That call to unselect should have called our installed
         * signal handler. */
        g_assert(g_curSelection == NULL);

        /* This is the most likely scenario, and will always do
         * what the user expects (see note below). */
        gtk_container_remove(GTK_CONTAINER(g_projectTree), itemToRemove);

        /* Put a message in the status bar. */
        message = g_strdup_printf("Removed \"%s\"", task->description->str);
        status_bar_set_message(message);
        g_free(message);
        return;
    }

    iter = gtk_container_children(GTK_CONTAINER(g_projectTree));
    while(iter != NULL)
    {
        taskLabel = GTK_LABEL(iter->data);
        gtk_label_get(taskLabel, &labelString);
        if(strcmp(labelString, task->description->str) == 0)
        {
            gchar* message;

            /* NOTE: This will remove the first occurrence of
             * a task with the same description, which may NOT
             * be what the user intended. */
            /* FIXME: Need to find a better solution (probably store
             * a mapping of task->widget). */
            gtk_container_remove(GTK_CONTAINER(g_projectTree),
                                 GTK_WIDGET(iter->data));

            message = g_strdup_printf("Removed \"%s\"",
                                      task->description->str);
            status_bar_set_message(message);
            g_free(message);
            return;
        }

        iter = g_list_next(iter);
    }
}

static void
callback_select_project(GtkTreeItem *item,
                        Project* project)
{
    g_assert(item != NULL);

    /* This won't always be valid, but while we only have one
     * project in the task_tree, this has better be true! */
    g_assert(g_project == project);
    g_assert(project != NULL);

    /* Remember the selection, and set the status bar. */
    g_task = NULL;
    g_curSelection = GTK_WIDGET(item);
    status_bar_set_project(g_project);
}

static void
callback_select_task(GtkTreeItem *item,
                     Task* task)
{
    g_assert(item != NULL);
    g_assert(task != NULL);

    /* Remember the selection, and set the status bar. */
    g_task = task;
    g_curSelection = GTK_WIDGET(item);
    status_bar_set_task(task);
}

static void
callback_deselect(GtkTreeItem *item,
                  gpointer dummy)
{
    g_assert(dummy == NULL);

    /* The selected item has been deselected.  Remember this and
     * clear the status bar. */
    g_curSelection = NULL;
    g_task = NULL;
    status_bar_set_message("");
}
